use std::{ffi::CString, sync::Arc};

use crate::{
    commands::command_base::{CommandDeserializer, DeserializedCommand},
    enums::Model,
};

pub const DESERIALIZE_PRODUCT_IDENTIFIER_RAW_NAME: &str = "_pin";

#[derive(Debug)]
pub struct ProductIdentifierCommand {
    pub product_identifier: String,
    pub model: Model,
}

impl DeserializedCommand for ProductIdentifierCommand {
    fn raw_name(&self) -> &'static str {
        DESERIALIZE_PRODUCT_IDENTIFIER_RAW_NAME
    }

    fn apply_to_state(&self, state: &mut crate::state::AtemState) {
        state.info.product_identifier = Some(self.product_identifier.clone());
        state.info.model = self.model.clone();

        match state.info.model {
            Model::TwoME
            | Model::TwoME4K
            | Model::TwoMEBS4K
            | Model::Constellation
            | Model::Constellation8K
            | Model::ConstellationHD4ME
            | Model::Constellation4K4ME => {
                state.info.power = vec![false, false];
            }
            _ => {
                state.info.power = vec![false];
            }
        }
    }
}

#[derive(Default)]
pub struct ProductIdentifierCommandDeserializer {}

impl CommandDeserializer for ProductIdentifierCommandDeserializer {
    fn deserialize(&self, buffer: &[u8]) -> std::sync::Arc<dyn DeserializedCommand> {
        let null_byte_index = buffer
            .iter()
            .position(|byte| *byte == b'\0')
            .expect("No null byte");
        let product_identifier =
            CString::from_vec_with_nul(buffer[..(null_byte_index + 1)].to_vec())
                .expect("Malformed string");
        let model = buffer[40];

        Arc::new(ProductIdentifierCommand {
            product_identifier: product_identifier
                .to_str()
                .expect("Invalid rust string")
                .to_string(),
            model: model.into(),
        })
    }
}
